package com.sun.xml.internal.ws.server;

import java.net.MalformedURLException;
import com.sun.xml.internal.ws.wsdl.writer.DocumentLocationResolver;
import com.sun.xml.internal.ws.util.RuntimeVersion;
import java.util.Iterator;
import com.sun.xml.internal.ws.wsdl.writer.WSDLPatcher;
import com.sun.xml.internal.ws.api.server.DocumentAddressResolver;
import com.sun.xml.internal.ws.api.server.PortAddressResolver;
import javax.xml.stream.XMLStreamWriter;
import com.sun.xml.internal.ws.util.xml.XMLStreamReaderToXMLStreamWriter;
import com.sun.xml.internal.ws.api.streaming.XMLStreamWriterFactory;
import java.io.OutputStream;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamException;
import java.io.IOException;
import javax.xml.ws.WebServiceException;
import java.util.HashSet;
import com.sun.xml.internal.ws.wsdl.parser.ParserUtil;
import com.sun.xml.internal.ws.wsdl.parser.WSDLConstants;
import com.sun.xml.internal.ws.streaming.XMLStreamReaderUtil;
import java.util.Set;
import java.net.URL;
import com.sun.xml.internal.ws.wsdl.SDDocumentResolver;
import com.sun.istack.internal.Nullable;
import com.sun.xml.internal.ws.api.server.SDDocumentFilter;
import java.util.List;
import javax.xml.namespace.QName;
import com.sun.xml.internal.ws.api.server.SDDocument;
import com.sun.xml.internal.ws.api.server.SDDocumentSource;

public class SDDocumentImpl extends SDDocumentSource implements SDDocument
{
    private static final String NS_XSD = "http://www.w3.org/2001/XMLSchema";
    private static final QName SCHEMA_INCLUDE_QNAME;
    private static final QName SCHEMA_IMPORT_QNAME;
    private static final QName SCHEMA_REDEFINE_QNAME;
    private static final String VERSION_COMMENT;
    private final QName rootName;
    private final SDDocumentSource source;
    @Nullable
    List<SDDocumentFilter> filters;
    @Nullable
    SDDocumentResolver sddocResolver;
    private final URL url;
    private final Set<String> imports;
    
    public static SDDocumentImpl create(SDDocumentSource src, QName serviceName, QName portTypeName) {
        URL systemId = src.getSystemId();
        try {
            XMLStreamReader reader = src.read();
            try {
                XMLStreamReaderUtil.nextElementContent(reader);
                QName rootName = reader.getName();
                if (rootName.equals(WSDLConstants.QNAME_SCHEMA)) {
                    String tns = ParserUtil.getMandatoryNonEmptyAttribute(reader, "targetNamespace");
                    Set<String> importedDocs2 = new HashSet();
                    while (XMLStreamReaderUtil.nextContent(reader) != 8) {
                        if (reader.getEventType() != 1) {
                            continue;
                        }
                        QName name2 = reader.getName();
                        if (SDDocumentImpl.SCHEMA_INCLUDE_QNAME.equals(name2) || SDDocumentImpl.SCHEMA_IMPORT_QNAME.equals(name2) || SDDocumentImpl.SCHEMA_REDEFINE_QNAME.equals(name2)) {
                            String importedDoc2 = reader.getAttributeValue(null, "schemaLocation");
                            if (importedDoc2 != null) {
                                importedDocs2.add(new URL(src.getSystemId(), importedDoc2).toString());
                            }
                            else {
                                continue;
                            }
                        }
                        else {
                            continue;
                        }
                    }
                    return new SDDocumentImpl.SchemaImpl(rootName, systemId, src, tns, importedDocs2);
                }
                if (rootName.equals(WSDLConstants.QNAME_DEFINITIONS)) {
                    String tns = ParserUtil.getMandatoryNonEmptyAttribute(reader, "targetNamespace");
                    boolean hasPortType = false;
                    boolean hasService = false;
                    Set<String> importedDocs = new HashSet();
                    Set<QName> allServices = new HashSet();
                    while (XMLStreamReaderUtil.nextContent(reader) != 8) {
                        if (reader.getEventType() != 1) {
                            continue;
                        }
                        QName name = reader.getName();
                        if (WSDLConstants.QNAME_PORT_TYPE.equals(name)) {
                            String pn = ParserUtil.getMandatoryNonEmptyAttribute(reader, "name");
                            if (portTypeName != null && portTypeName.getLocalPart().equals(pn) && portTypeName.getNamespaceURI().equals(tns)) {
                                hasPortType = true;
                            }
                            else {
                                continue;
                            }
                        }
                        else if (WSDLConstants.QNAME_SERVICE.equals(name)) {
                            String sn = ParserUtil.getMandatoryNonEmptyAttribute(reader, "name");
                            QName sqn = new QName(tns, sn);
                            allServices.add(sqn);
                            if (serviceName.equals(sqn)) {
                                hasService = true;
                            }
                            else {
                                continue;
                            }
                        }
                        else if (WSDLConstants.QNAME_IMPORT.equals(name)) {
                            String importedDoc = reader.getAttributeValue(null, "location");
                            if (importedDoc != null) {
                                importedDocs.add(new URL(src.getSystemId(), importedDoc).toString());
                            }
                            else {
                                continue;
                            }
                        }
                        else if (SDDocumentImpl.SCHEMA_INCLUDE_QNAME.equals(name) || SDDocumentImpl.SCHEMA_IMPORT_QNAME.equals(name) || SDDocumentImpl.SCHEMA_REDEFINE_QNAME.equals(name)) {
                            String importedDoc = reader.getAttributeValue(null, "schemaLocation");
                            if (importedDoc != null) {
                                importedDocs.add(new URL(src.getSystemId(), importedDoc).toString());
                            }
                            else {
                                continue;
                            }
                        }
                        else {
                            continue;
                        }
                    }
                    return new SDDocumentImpl.WSDLImpl(rootName, systemId, src, tns, hasPortType, hasService, importedDocs, allServices);
                }
                return new SDDocumentImpl(rootName, systemId, src);
            }
            finally {
                reader.close();
            }
        }
        catch (WebServiceException e) {
            throw new ServerRtException("runtime.parser.wsdl", new Object[] { systemId, e });
        }
        catch (IOException e2) {
            throw new ServerRtException("runtime.parser.wsdl", new Object[] { systemId, e2 });
        }
        catch (XMLStreamException e3) {
            throw new ServerRtException("runtime.parser.wsdl", new Object[] { systemId, e3 });
        }
    }
    
    protected SDDocumentImpl(QName rootName, URL url, SDDocumentSource source) {
        this(rootName, url, source, new HashSet());
    }
    
    protected SDDocumentImpl(QName rootName, URL url, SDDocumentSource source, Set<String> imports) {
        if (url == null) {
            throw new IllegalArgumentException("Cannot construct SDDocument with null URL.");
        }
        this.rootName = rootName;
        this.source = source;
        this.url = url;
        this.imports = imports;
        return;
    }
    
    void setFilters(List<SDDocumentFilter> filters) {
        this.filters = filters;
    }
    
    void setResolver(SDDocumentResolver sddocResolver) {
        this.sddocResolver = sddocResolver;
    }
    
    public QName getRootName() {
        return this.rootName;
    }
    
    public boolean isWSDL() {
        return false;
    }
    
    public boolean isSchema() {
        return false;
    }
    
    public URL getURL() {
        return this.url;
    }
    
    public XMLStreamReader read(XMLInputFactory xif) throws IOException, XMLStreamException {
        return this.source.read(xif);
    }
    
    public XMLStreamReader read() throws IOException, XMLStreamException {
        return this.source.read();
    }
    
    public URL getSystemId() {
        return this.url;
    }
    
    public Set<String> getImports() {
        return this.imports;
    }
    
    public void writeTo(OutputStream os) throws IOException {
        XMLStreamWriter w = null;
        try {
            w = XMLStreamWriterFactory.create(os, "UTF-8");
            w.writeStartDocument("UTF-8", "1.0");
            new XMLStreamReaderToXMLStreamWriter().bridge(this.source.read(), w);
            w.writeEndDocument();
        }
        catch (XMLStreamException e) {
            IOException ioe = new IOException(e.getMessage());
            ioe.initCause(e);
            throw ioe;
        }
        finally {
            try {
                if (w != null) {
                    w.close();
                }
            }
            catch (XMLStreamException e2) {
                IOException ioe2 = new IOException(e2.getMessage());
                ioe2.initCause(e2);
                throw ioe2;
            }
        }
    }
    
    public void writeTo(PortAddressResolver portAddressResolver, DocumentAddressResolver resolver, OutputStream os) throws IOException {
        XMLStreamWriter w = null;
        try {
            w = XMLStreamWriterFactory.create(os, "UTF-8");
            w.writeStartDocument("UTF-8", "1.0");
            this.writeTo(portAddressResolver, resolver, w);
            w.writeEndDocument();
        }
        catch (XMLStreamException e) {
            IOException ioe = new IOException(e.getMessage());
            ioe.initCause(e);
            throw ioe;
        }
        finally {
            try {
                if (w != null) {
                    w.close();
                }
            }
            catch (XMLStreamException e2) {
                IOException ioe2 = new IOException(e2.getMessage());
                ioe2.initCause(e2);
                throw ioe2;
            }
        }
    }
    
    public void writeTo(PortAddressResolver portAddressResolver, DocumentAddressResolver resolver, XMLStreamWriter out) throws XMLStreamException, IOException {
        if (this.filters != null) {
            for (SDDocumentFilter f : this.filters) {
                out = f.filter(this, out);
            }
        }
        XMLStreamReader xsr = this.source.read();
        try {
            out.writeComment(SDDocumentImpl.VERSION_COMMENT);
            new WSDLPatcher(portAddressResolver, new SDDocumentImpl.DocumentLocationResolverImpl(resolver)).bridge(xsr, out);
        }
        finally {
            xsr.close();
        }
    }
    
    static {
        SCHEMA_INCLUDE_QNAME = new QName("http://www.w3.org/2001/XMLSchema", "include");
        SCHEMA_IMPORT_QNAME = new QName("http://www.w3.org/2001/XMLSchema", "import");
        SCHEMA_REDEFINE_QNAME = new QName("http://www.w3.org/2001/XMLSchema", "redefine");
        VERSION_COMMENT = " Published by JAX-WS RI (http://jax-ws.java.net). RI's version is " + RuntimeVersion.VERSION + ". ";
    }
    
    private static final class SchemaImpl extends SDDocumentImpl implements SDDocument.Schema
    {
        private final String targetNamespace;
        
        public SchemaImpl(QName rootName, URL url, SDDocumentSource source, String targetNamespace, Set<String> imports) {
            super(rootName, url, source, imports);
            this.targetNamespace = targetNamespace;
        }
        
        public String getTargetNamespace() {
            return this.targetNamespace;
        }
        
        @Override
        public boolean isSchema() {
            return true;
        }
    }
    
    private static final class WSDLImpl extends SDDocumentImpl implements SDDocument.WSDL
    {
        private final String targetNamespace;
        private final boolean hasPortType;
        private final boolean hasService;
        private final Set<QName> allServices;
        
        public WSDLImpl(QName rootName, URL url, SDDocumentSource source, String targetNamespace, boolean hasPortType, boolean hasService, Set<String> imports, Set<QName> allServices) {
            super(rootName, url, source, imports);
            this.targetNamespace = targetNamespace;
            this.hasPortType = hasPortType;
            this.hasService = hasService;
            this.allServices = allServices;
        }
        
        public String getTargetNamespace() {
            return this.targetNamespace;
        }
        
        public boolean hasPortType() {
            return this.hasPortType;
        }
        
        public boolean hasService() {
            return this.hasService;
        }
        
        public Set<QName> getAllServices() {
            return this.allServices;
        }
        
        @Override
        public boolean isWSDL() {
            return true;
        }
    }
    
    private class DocumentLocationResolverImpl implements DocumentLocationResolver
    {
        private DocumentAddressResolver delegate;
        
        DocumentLocationResolverImpl(DocumentAddressResolver delegate) {
            this.delegate = delegate;
        }
        
        public String getLocationFor(String namespaceURI, String systemId) {
            if (SDDocumentImpl.this.sddocResolver == null) {
                return systemId;
            }
            try {
                URL ref = new URL(SDDocumentImpl.this.getURL(), systemId);
                SDDocument refDoc = SDDocumentImpl.this.sddocResolver.resolve(ref.toExternalForm());
                if (refDoc == null) {
                    return systemId;
                }
                return this.delegate.getRelativeAddressFor(SDDocumentImpl.this, refDoc);
            }
            catch (MalformedURLException mue) {
                return null;
            }
        }
    }
}
